@@ -59,7 +59,7 @@
 		      * require consumer to be updated (as long as this number
 		      * is zero, the API is not considered stable and can
 		      * change without a bump of the major version) */
-#define MINVERSION 4 /* API compatible, ABI may change, functional
+#define MINVERSION 5 /* API compatible, ABI may change, functional
 		      * enhancements only, consumer can be left unchanged if
 		      * enhancements are not considered */
 #define PATCHLEVEL 0 /* API / ABI compatible, no functional changes, no
@@ -68,6 +68,7 @@
 /* remove once in if_alg.h */
 #define ALG_SET_AEAD_ASSOCLEN		4
 #define ALG_SET_AEAD_AUTHSIZE		5
+#define ALG_SET_RNG_SEED		6
 
 /* remove once in socket.h */
 #ifndef AF_ALG
@@ -247,7 +248,7 @@ static int __kcapi_common_getinfo(struct kcapi_handle *handle,
 	struct rtattr *tb[CRYPTOCFGA_MAX+1];
 	struct rtattr *rta;
 
-	/* AunsignedF_NETLINK specific */
+	/* AF_NETLINK specific */
 	struct sockaddr_nl nl;
 	int sd = 0;
 	socklen_t addr_len;
@@ -525,8 +526,10 @@ void kcapi_versionstring(char *buf, size_t buflen)
  * The function allocates memory for @newiv in case the return code indicates
  * success. The consumer must free the memory after use.
  *
- * Return: 0 for success; -ERANGE when the provided IV already satisfies the
- *	   the alignment requirement; < 0 for any other errors
+ * Return: 0 for success;
+ *	   -ERANGE when the provided IV already satisfies the
+ *	   the alignment requirement;
+ *	   < 0 for any other errors
  */
 int kcapi_pad_iv(struct kcapi_handle *handle,
 		 const unsigned char *iv, size_t ivlen,
@@ -590,7 +593,8 @@ int kcapi_cipher_destroy(struct kcapi_handle *handle)
  * @key: key buffer - input
  * @keylen: length of key buffer - input
  *
- * Return: 0 upon success; < 0 in case of error
+ * Return: 0 upon success;
+ *	   < 0 in case of error
  */
 int kcapi_cipher_setkey(struct kcapi_handle *handle,
 			const unsigned char *key, size_t keylen)
@@ -604,10 +608,12 @@ int kcapi_cipher_setkey(struct kcapi_handle *handle,
  * @iv: buffer holding the IV (may be NULL if IV is not needed) - input
  * @ivlen: length of iv (should be zero if iv is NULL) - input
  *
- * Return: 0 upon success; < 0 in case of an error
- *
  * This function requires IV to be exactly IV size. The function verifies
  * the IV size to avoid unnecessary kernel round trips.
+ *
+ * Return: 0 upon success;
+ *	   < 0 in case of an error
+ *
  */
 int kcapi_cipher_setiv(struct kcapi_handle *handle,
 		       const unsigned char *iv, size_t ivlen)
@@ -627,8 +633,8 @@ int kcapi_cipher_setiv(struct kcapi_handle *handle,
  * ciphertext pointers. That would mean that after the encryption operation,
  * the plaintext is overwritten with the ciphertext.
  *
- * Return: number of bytes encrypted upon success; < 0 in case of error with
- *	   errno set
+ * Return: number of bytes encrypted upon success;
+ *	   < 0 in case of error with errno set
  */
 ssize_t kcapi_cipher_encrypt(struct kcapi_handle *handle,
 			     const unsigned char *in, size_t inlen,
@@ -636,7 +642,7 @@ ssize_t kcapi_cipher_encrypt(struct kcapi_handle *handle,
 {
 	struct iovec iov;
 	ssize_t ret = 0;
-	int bs = handle->info.blocksize;
+	unsigned int bs = handle->info.blocksize;
 
 	if (!in || !inlen || !out || !outlen) {
 		fprintf(stderr,
@@ -647,7 +653,7 @@ ssize_t kcapi_cipher_encrypt(struct kcapi_handle *handle,
 	/* require properly sized output data size */
 	if (outlen < ((inlen + bs - 1) / bs * bs)) {
 		fprintf(stderr,
-			"Symmetric Encryption: Ciphertext buffer (%lu) is not plaintext buffer (%lu) rounded up to multiple of block size %d\n",
+			"Symmetric Encryption: Ciphertext buffer (%lu) is not plaintext buffer (%lu) rounded up to multiple of block size %u\n",
 			(unsigned long) outlen, (unsigned long)inlen, bs);
 		return -EINVAL;
 	}
@@ -684,8 +690,8 @@ ssize_t kcapi_cipher_encrypt(struct kcapi_handle *handle,
  * ciphertext pointers. That would mean that after the encryption operation,
  * the ciphertext is overwritten with the plaintext.
  *
- * Return: number of bytes decrypted upon success; < 0 in case of error with
- *	   errno set
+ * Return: number of bytes decrypted upon success;
+ *	   < 0 in case of error with errno set
  */
 ssize_t kcapi_cipher_decrypt(struct kcapi_handle *handle,
 			     const unsigned char *in, size_t inlen,
@@ -703,7 +709,7 @@ ssize_t kcapi_cipher_decrypt(struct kcapi_handle *handle,
 	/* require properly sized output data size */
 	if (inlen % handle->info.blocksize) {
 		fprintf(stderr,
-			"Symmetric Decryption: Ciphertext buffer is not multiple of block size %d\n",
+			"Symmetric Decryption: Ciphertext buffer is not multiple of block size %u\n",
 			handle->info.blocksize);
 		return -EINVAL;
 	}
@@ -757,8 +763,8 @@ ssize_t kcapi_cipher_decrypt(struct kcapi_handle *handle,
  * kcapi_cipher_stream_op() can be mixed, even by multiple threads of an
  * application.
  *
- * Return: number of bytes sent to the kernel upon success; < 0 in case of error
- *	   with errno set
+ * Return: number of bytes sent to the kernel upon success;
+ *	   < 0 in case of error with errno set
  */
 ssize_t kcapi_cipher_stream_init_enc(struct kcapi_handle *handle,
 				     struct iovec *iov, size_t iovlen)
@@ -788,8 +794,8 @@ ssize_t kcapi_cipher_stream_init_enc(struct kcapi_handle *handle,
  * kcapi_cipher_stream_op() can be mixed, even by multiple threads of an
  * application.
  *
- * Return: number of bytes sent to the kernel upon success; < 0 in case of error
- *	   with errno set
+ * Return: number of bytes sent to the kernel upon success;
+ *	   < 0 in case of error with errno set
  */
 ssize_t kcapi_cipher_stream_init_dec(struct kcapi_handle *handle,
 				     struct iovec *iov, size_t iovlen)
@@ -816,8 +822,8 @@ ssize_t kcapi_cipher_stream_init_dec(struct kcapi_handle *handle,
  * where one thread sends data to be processed and one thread picks up data
  * processed by the cipher operation.
  *
- * Return: number of bytes sent to the kernel upon success; < 0 in case of error
- *	   with errno set
+ * Return: number of bytes sent to the kernel upon success;
+ *	   < 0 in case of error with errno set
  */
 ssize_t kcapi_cipher_stream_update(struct kcapi_handle *handle,
 				   struct iovec *iov, size_t iovlen)
@@ -843,8 +849,8 @@ ssize_t kcapi_cipher_stream_update(struct kcapi_handle *handle,
  * will only be filled up to the maximum number of full block sizes that fit
  * into the buffer.
  *
- * Return: number of bytes obtained from the kernel upon success; < 0 in case
- *	   of error with errno set
+ * Return: number of bytes obtained from the kernel upon success;
+ *	   < 0 in case of error with errno set
  */
 ssize_t kcapi_cipher_stream_op(struct kcapi_handle *handle,
 			       struct iovec *iov, size_t iovlen)
@@ -861,9 +867,10 @@ ssize_t kcapi_cipher_stream_op(struct kcapi_handle *handle,
  * kcapi_cipher_ivsize() - return size of IV required for cipher
  * @handle: cipher handle - input
  *
- * Return: > 0 specifying the IV size; 0 on error
+ * Return: > 0 specifying the IV size;
+ *	   0 on error
  */
-int kcapi_cipher_ivsize(struct kcapi_handle *handle)
+unsigned int kcapi_cipher_ivsize(struct kcapi_handle *handle)
 {
 	return handle->info.ivsize;
 }
@@ -872,9 +879,10 @@ int kcapi_cipher_ivsize(struct kcapi_handle *handle)
  * kcapi_cipher_blocksize() - return size of one block of the cipher
  * @handle: cipher handle - input
  *
- * Return: > 0 specifying the block size; 0 on error
+ * Return: > 0 specifying the block size;
+ *	   0 on error
  */
-int kcapi_cipher_blocksize(struct kcapi_handle *handle)
+unsigned int kcapi_cipher_blocksize(struct kcapi_handle *handle)
 {
 	return handle->info.blocksize;
 }
@@ -895,7 +903,8 @@ int kcapi_cipher_blocksize(struct kcapi_handle *handle)
  * This function initializes an AEAD cipher handle and establishes the
  * connection to the kernel.
  *
- * Return: 0 upon success; -ENOENT - algorithm not available;
+ * Return: 0 upon success;
+ *	   -ENOENT - algorithm not available;
  *	   -EOPNOTSUPP - AF_ALG family not available;
  *	   -EINVAL - accept syscall failed
  */
@@ -921,7 +930,8 @@ int kcapi_aead_destroy(struct kcapi_handle *handle)
  * @key: key buffer - input
  * @keylen: length of key buffer - input
  *
- * Return: 0 upon success; < 0 in case of error
+ * Return: 0 upon success;
+ *	   < 0 in case of error
  */
 int kcapi_aead_setkey(struct kcapi_handle *handle,
 		      const unsigned char *key, size_t keylen)
@@ -954,13 +964,14 @@ int kcapi_aead_setiv(struct kcapi_handle *handle,
  * Set the authentication tag size needed for encryption operation. The tag is
  * created during encryption operation with the size provided with this call.
  *
- * Return: 0 upon success; < 0 in case of error with errno set
+ * Return: 0 upon success;
+ *	   < 0 in case of error with errno set
  */
 int kcapi_aead_settaglen(struct kcapi_handle *handle, size_t taglen)
 {
 	handle->aead.tag = NULL;
 	handle->aead.taglen = taglen;
-	if (setsockopt(handle->opfd, SOL_ALG, ALG_SET_AEAD_AUTHSIZE,
+	if (setsockopt(handle->tfmfd, SOL_ALG, ALG_SET_AEAD_AUTHSIZE,
 		       NULL, taglen) == -1)
 		return -EINVAL;
 
@@ -998,8 +1009,8 @@ void kcapi_aead_setassoclen(struct kcapi_handle *handle, size_t assoclen)
  * kcapi_aead_getdata() to obtain the resulting ciphertext and authentication
  * tag references.
  *
- * Return: number of bytes encrypted upon success; < 0 in case of error with
- *	   errno set
+ * Return: number of bytes encrypted upon success;
+ *	   < 0 in case of error with errno set
  */
 ssize_t kcapi_aead_encrypt(struct kcapi_handle *handle,
 			   const unsigned char *in, size_t inlen,
@@ -1028,6 +1039,8 @@ ssize_t kcapi_aead_encrypt(struct kcapi_handle *handle,
 	}
 
 #if 0
+	/* using two syscall */
+	/* TODO make heuristic when one syscall is slower than four syscalls */
 	if (assoc && handle->aead.assoclen) {
 		iov[0].iov_base = (void*)(uintptr_t)assoc;
 		iov[0].iov_len = handle->aead.assoclen;
@@ -1138,8 +1151,8 @@ void kcapi_aead_getdata(struct kcapi_handle *handle,
  * If this function returns < 0 and errno is set to EBADMSG, an authentication
  * error is detected.
  *
- * Return: number of bytes decrypted upon success; < 0 in case of error with
- *	   errno set
+ * Return: number of bytes decrypted upon success;
+ *	   < 0 in case of error with errno set
  */
 ssize_t kcapi_aead_decrypt(struct kcapi_handle *handle,
 			   const unsigned char *in, size_t inlen,
@@ -1149,7 +1162,7 @@ ssize_t kcapi_aead_decrypt(struct kcapi_handle *handle,
 	struct iovec iov[3];
 	ssize_t ret = 0;
 	size_t len = 0;
-	int bs = handle->info.blocksize;
+	unsigned int bs = handle->info.blocksize;
 
 	if (!in || !inlen || !out || !outlen || !tag || !handle->aead.taglen) {
 		fprintf(stderr,
@@ -1160,13 +1173,15 @@ ssize_t kcapi_aead_decrypt(struct kcapi_handle *handle,
 	/* require properly sized output data size */
 	if (outlen < _kcapi_aead_decrypt_outlen(handle, inlen)) {
 		fprintf(stderr,
-			"AEAD Decryption: Plaintext buffer (%lu) is not ciphertext buffer (%lu) reduced by tag length (%lu) routed up to multiple of block size %d\n",
+			"AEAD Decryption: Plaintext buffer (%lu) is not ciphertext buffer (%lu) reduced by tag length (%lu) routed up to multiple of block size %u\n",
 			(unsigned long)outlen, (unsigned long) inlen,
 			(unsigned long)handle->aead.taglen, bs);
 		return -EINVAL;
 	}
 
 #if 0
+	/* using two syscall */
+	/* TODO make heuristic when one syscall is slower than four syscalls */
 	if (assoc && handle->aead.assoclen) {
 		iov[0].iov_base = (void*)(uintptr_t)assoc;
 		iov[0].iov_len = handle->aead.assoclen;
@@ -1250,8 +1265,8 @@ ssize_t kcapi_aead_decrypt(struct kcapi_handle *handle,
  * the complete associated data must be provided, followed by (ii) the
  * plaintext.
  *
- * Return: number of bytes sent to the kernel upon success; < 0 in case of error
- *	   with errno set
+ * Return: number of bytes sent to the kernel upon success;
+ *	   < 0 in case of error with errno set
  */
 ssize_t kcapi_aead_stream_init_enc(struct kcapi_handle *handle,
 				   struct iovec *iov, size_t iovlen)
@@ -1289,8 +1304,8 @@ ssize_t kcapi_aead_stream_init_enc(struct kcapi_handle *handle,
  * the complete associated data must be provided, followed by (ii) the
  * plaintext. For decryption, also (iii) the tag value must be sent.
  *
- * Return: number of bytes sent to the kernel upon success; < 0 in case of error
- *	   with errno set
+ * Return: number of bytes sent to the kernel upon success;
+ *	   < 0 in case of error with errno set
  */
 ssize_t kcapi_aead_stream_init_dec(struct kcapi_handle *handle,
 				   struct iovec *iov, size_t iovlen)
@@ -1324,8 +1339,8 @@ ssize_t kcapi_aead_stream_init_dec(struct kcapi_handle *handle,
  * where one thread sends data to be processed and one thread picks up data
  * processed by the cipher operation.
  *
- * Return: number of bytes sent to the kernel upon success; < 0 in case of error
- *	   with errno set
+ * Return: number of bytes sent to the kernel upon success;
+ *	   < 0 in case of error with errno set
  */
 ssize_t kcapi_aead_stream_update(struct kcapi_handle *handle,
 				 struct iovec *iov, size_t iovlen)
@@ -1347,8 +1362,8 @@ ssize_t kcapi_aead_stream_update(struct kcapi_handle *handle,
  * exception that it marks the last data buffer before the cipher operation
  * is triggered. Typically, the tag value is provided with this call.
  *
- * Return: number of bytes sent to the kernel upon success; < 0 in case of error
- *	   with errno set
+ * Return: number of bytes sent to the kernel upon success;
+ *	   < 0 in case of error with errno set
  */
 ssize_t kcapi_aead_stream_update_last(struct kcapi_handle *handle,
 				      struct iovec *iov, size_t iovlen)
@@ -1371,8 +1386,8 @@ ssize_t kcapi_aead_stream_update_last(struct kcapi_handle *handle,
  * will only be filled up to the maximum number of full block sizes that fit
  * into the buffer.
  *
- * Return: number of bytes obtained from the kernel upon success; < 0 in case
- *	   of error with errno set
+ * Return: number of bytes obtained from the kernel upon success;
+ *	   < 0 in case of error with errno set
  */
 ssize_t kcapi_aead_stream_op(struct kcapi_handle *handle,
 			     struct iovec *iov, size_t iovlen)
@@ -1394,9 +1409,10 @@ ssize_t kcapi_aead_stream_op(struct kcapi_handle *handle,
  * kcapi_aead_ivsize() - return size of IV required for cipher
  * @handle: cipher handle - input
  *
- * Return: > 0 specifying the IV size; 0 on error
+ * Return: > 0 specifying the IV size;
+ *	   0 on error
  */
-int kcapi_aead_ivsize(struct kcapi_handle *handle)
+unsigned int kcapi_aead_ivsize(struct kcapi_handle *handle)
 {
 	return handle->info.ivsize;
 }
@@ -1405,9 +1421,10 @@ int kcapi_aead_ivsize(struct kcapi_handle *handle)
  * kcapi_aead_blocksize() - return size of one block of the cipher
  * @handle: cipher handle - input
  *
- * Return: > 0 specifying the block size; 0 on error
+ * Return: > 0 specifying the block size;
+ *	   0 on error
  */
-int kcapi_aead_blocksize(struct kcapi_handle *handle)
+unsigned int kcapi_aead_blocksize(struct kcapi_handle *handle)
 {
 	return handle->info.blocksize;
 }
@@ -1420,9 +1437,10 @@ int kcapi_aead_blocksize(struct kcapi_handle *handle)
  * be produced by the AEAD cipher. Smaller tag sizes may be chosen depending on
  * the AEAD cipher type.
  *
- * Return: > 0 specifying the block size; 0 on error
+ * Return: > 0 specifying the block size;
+ *	   0 on error
  */
-int kcapi_aead_authsize(struct kcapi_handle *handle)
+unsigned int kcapi_aead_authsize(struct kcapi_handle *handle)
 {
 	return handle->info.aead_maxauthsize;
 }
@@ -1458,7 +1476,8 @@ size_t kcapi_aead_outbuflen(struct kcapi_handle *handle,
  *
  * Caller must free iv.
  *
- * Return: 0 upon success; < 0 upon failure
+ * Return: 0 upon success;
+ *	   < 0 upon failure
  */
 int kcapi_aead_ccm_nonce_to_iv(const unsigned char *nonce, size_t noncelen,
 			       unsigned char **iv, size_t *ivlen)
@@ -1521,8 +1540,8 @@ int kcapi_md_destroy(struct kcapi_handle *handle)
  *
  * This call is applicable for keyed message digests.
  *
- *
- * Return: 0 upon success; < 0 in case of error
+ * Return: 0 upon success;
+ *	   < 0 in case of error
  */
 int kcapi_md_setkey(struct kcapi_handle *handle,
 		    const unsigned char *key, size_t keylen)
@@ -1548,7 +1567,8 @@ static ssize_t _kcapi_md_update(struct kcapi_handle *handle,
  * @buffer: holding the data to add to the message digest - input
  * @len: buffer length - input
  *
- * Return: 0 upon success; < 0 in case of error
+ * Return: 0 upon success;
+ *	   < 0 in case of error
  */
 ssize_t kcapi_md_update(struct kcapi_handle *handle,
 		    const unsigned char *buffer, size_t len)
@@ -1565,7 +1585,7 @@ static ssize_t _kcapi_md_final(struct kcapi_handle *handle,
 
 	if (len < (unsigned long)handle->info.hash_digestsize) {
 		fprintf(stderr,
-			"Message digest: output buffer too small (seen %lu - required %d)\n",
+			"Message digest: output buffer too small (seen %lu - required %u)\n",
 			(unsigned long)len,
 			handle->info.hash_digestsize);
 		return -EINVAL;
@@ -1595,7 +1615,8 @@ static ssize_t _kcapi_md_final(struct kcapi_handle *handle,
  * @buffer: filled with the message digest - output
  * @len: buffer length - input
  *
- * Return: size of message digest upon success; -EIO - data cannot be obtained
+ * Return: size of message digest upon success;
+ *	   -EIO - data cannot be obtained;
  * 	   -ENOMEM - buffer is too small for the complete message digest,
  * 	   the buffer is filled with the truncated message digest
  */
@@ -1621,7 +1642,8 @@ ssize_t kcapi_md_final(struct kcapi_handle *handle,
  * The message digest handle must have been initialized, potentially by also
  * setting the key using the generic message digest API functions.
  *
- * Return: size of message digest upon success; -EIO - data cannot be obtained
+ * Return: size of message digest upon success;
+ *	   -EIO - data cannot be obtained;
  * 	   -ENOMEM - buffer is too small for the complete message digest,
  * 	   the buffer is filled with the truncated message digest
  */
@@ -1640,7 +1662,7 @@ ssize_t kcapi_md_digest(struct kcapi_handle *handle,
 
 	if (outlen < (unsigned int)handle->info.hash_digestsize) {
 		fprintf(stderr,
-			"Message digest: output buffer too small (seen %lu - required %d)\n",
+			"Message digest: output buffer too small (seen %lu - required %u)\n",
 			(unsigned long)outlen,
 			handle->info.hash_digestsize);
 		return -EINVAL;
@@ -1670,9 +1692,10 @@ ssize_t kcapi_md_digest(struct kcapi_handle *handle,
  * function invocation to determine the right memory size to be allocated for
  * this call.
  *
- * Return: > 0 specifying the block size; 0 on error
+ * Return: > 0 specifying the block size;
+ *	   0 on error
  */
-int kcapi_md_digestsize(struct kcapi_handle *handle)
+unsigned int kcapi_md_digestsize(struct kcapi_handle *handle)
 {
 	return handle->info.hash_digestsize;
 }
@@ -1710,14 +1733,33 @@ int kcapi_rng_destroy(struct kcapi_handle *handle)
 	return _kcapi_handle_destroy(handle);
 }
 
+/**
+ * kcapi_rng_seed() - Seed the RNG
+ * @handle: cipher handle - input
+ * @seed: seed data - input
+ * @seedlen: size of @seed
+ *
+ * Return: 0 upon success;
+ * 	   < 0 upon error
+ */
+int kcapi_rng_seed(struct kcapi_handle *handle, unsigned char *seed,
+		   size_t seedlen)
+{
+	if (setsockopt(handle->tfmfd, SOL_ALG, ALG_SET_RNG_SEED,
+		       seed, seedlen) == -1)
+		return -EINVAL;
+
+	return 0;
+}
+
 /**
  * kcapi_rng_generate() - generate a random number
  * @handle: cipher handle - input
  * @buffer: filled with the random number - output
  * @len: buffer length - input
  *
- * Return: size of random number generated upon success; -EIO - data cannot be
- *	   obtained
+ * Return: size of random number generated upon success;
+ *	   -EIO - data cannot be obtained
  */
 ssize_t kcapi_rng_generate(struct kcapi_handle *handle,
 			   unsigned char *buffer, size_t len)
