@@ -124,7 +124,7 @@ static int hex2bin_m(const char *hex, size_t hexlen,
 	return 0;
 }
 
-static int aux_test_rng(char *name)
+static int aux_test_rng(char *name, unsigned char *seed, size_t seedlen)
 {
 	struct kcapi_handle handle;
 #define RNGOUTBUF 150
@@ -137,6 +137,15 @@ static int aux_test_rng(char *name)
                 return 1;
         }
 
+	if (seed && seedlen) {
+		ret = kcapi_rng_seed(&handle, seed, seedlen);
+		if (0 > ret) {
+			printf("Failure to seed RNG %d\n", (int)ret);
+			kcapi_rng_destroy(&handle);
+			return 1;
+		}
+	}
+
 	ret = kcapi_rng_generate(&handle, outbuf, RNGOUTBUF);
 	if (0 > ret) {
 		printf("Failure to generate random numbers %d\n",
@@ -207,13 +216,31 @@ static int auxiliary_tests(void)
 	kcapi_md_destroy(&handle);
 
 
-	if (aux_test_rng("drbg_nopr_hmac_sha256"))
+	if (aux_test_rng("drbg_nopr_hmac_sha256", NULL, 0))
 		ret++;
-	if (aux_test_rng("drbg_nopr_sha1"))
+	if (aux_test_rng("drbg_nopr_sha1", NULL, 0))
 		ret++;
-	if (aux_test_rng("drbg_nopr_ctr_aes256"))
+	if (aux_test_rng("drbg_nopr_ctr_aes256", NULL, 0))
 		ret++;
-	if (aux_test_rng("ansi_cprng"))
+
+	/* X9.31 RNG must require seed */
+	printf("X9.31 missing seeding: ");
+	if (!aux_test_rng("ansi_cprng", NULL, 0))
+		ret++;
+	/* X9.31 seed too short */
+	printf("X9.31 insufficient seeding: ");
+	if (!aux_test_rng("ansi_cprng",
+			  (unsigned char *)
+			  "\x00\x01\x02\x03\x04\x05\x06\x07\x08"
+			  "\x00\x01\x02\x03\x04\x05\x06\x07\x08", 16))
+		ret++;
+	/* X9.31 seed right sized short */
+	if (aux_test_rng("ansi_cprng",
+			 (unsigned char *)
+			 "\x00\x01\x02\x03\x04\x05\x06\x07\x08"
+			 "\x00\x01\x02\x03\x04\x05\x06\x07\x08"
+			 "\x00\x01\x02\x03\x04\x05\x06\x07\x08"
+			 "\x00\x01\x02\x03\x04\x05\x06\x07\x08", 32))
 		ret++;
 
 	return 0;
